<?php
/**
 * プラグイン作りでどのクラスからも参照する汎用クラス
 *
 * @package qa_heatmap
 */

class QAHM_Base {
	/**
	 * wordpressのget_option関数をqahm用に使いやすくした関数
	 */
	public function wrap_get_option( $option, $default = false ) {
		if ( $default === false ) {
			foreach ( QAHM_OPTIONS as $key => $value ) {
				if ( $option === $key ) {
					$default = $value;
					break;
				}
			}
			foreach ( QAHM_DB_OPTIONS as $key => $value ) {
				if ( $option === $key ) {
					$default = $value;
					break;
				}
			}
		}
		return get_option( QAHM_OPTION_PREFIX . $option, $default );
	}

	/**
	 * wordpressのupdate_option関数をqahm用に使いやすくした関数
	 */
	public function wrap_update_option( $option, $value ) {
		return update_option( QAHM_OPTION_PREFIX . $option, $value );
	}

	/**
	 * アクセス権限判定
	 */
	public function check_access_role() {
		if ( ! is_user_logged_in() ) {
			return false;
		}
		$user   = wp_get_current_user();
		$roles  = (array) $user->roles;
		$role   = $roles[0];
		$access = $this->wrap_get_option( 'access_role', 'administrator' );

		switch ( $role ) {
			case 'administrator':
				return ( 'administrator' === $access || 'editor' === $access ) ? true : false;
			case 'editor':
				return ( 'editor' === $access ) ? true : false;
			default:
				return false;
		}
	}

	/**
	 * プラグインのメインファイルパスを取得
	 */
	public function get_plugin_main_file_path() {
		return dirname( __FILE__ ) . '/' . QAHM_NAME . '.php';
	}

	/**
	 * jsディレクトリのパスを取得
	 */
	public function get_js_dir_path() {
		return plugin_dir_path( __FILE__ ) . 'js/';
	}

	/**
	 * jsディレクトリのURLを取得
	 */
	public function get_js_dir_url() {
		return plugin_dir_url( __FILE__ ) . 'js/';
	}

	/**
	 * cssディレクトリのパスを取得
	 */
	public function get_css_dir_path() {
		return plugin_dir_path( __FILE__ ) . 'css/';
	}

	/**
	 * cssディレクトリのURLを取得
	 */
	public function get_css_dir_url() {
		return plugin_dir_url( __FILE__ ) . 'css/';
	}

	/**
	 * imgディレクトリのパスを取得
	 */
	public function get_img_dir_path() {
		return plugin_dir_path( __FILE__ ) . 'img/';
	}

	/**
	 * imgディレクトリのURLを取得
	 */
	public function get_img_dir_url() {
		return plugin_dir_url(__FILE__) . 'img/';
	}

	/**
	 * dataディレクトリのパスを取得
	 * 引数にdataディレクトリからのパスを入力することにより、
	 * dataディレクトリからの相対パスを取得することができる。
	 * 
	 * なおこの関数では念のためディレクトリの存在チェック＆mkdirも行うが、
	 * 相対パスに深い階層を指定してもmkdirされるのは最後の階層のみであり
	 * 道中の階層にはmkdirされない。その点には注意
	 */
	public function get_data_dir_path( $data_rel_path = '' ) {
		global $wp_filesystem;
		$path = $wp_filesystem->wp_content_dir() . QAHM_TEXT_DOMAIN . '-data/';
		if ( ! $wp_filesystem->exists( $path ) ) {
			$wp_filesystem->mkdir( $path );
		}

		if ( $data_rel_path ) {
			$path .= $data_rel_path;
			if ( substr( $path, -1 ) !== '/' ) {
				$path .= '/';
			}

			if ( ! $wp_filesystem->exists( $path ) ) {
				$wp_filesystem->mkdir( $path );
			}
		}

		return $path;
	}

	/**
	 * dataディレクトリのURLを取得
	 */
	public function get_data_dir_url( $data_rel_path = '' ) {
		$path = content_url() . '/' . QAHM_TEXT_DOMAIN . '-data/';

		if ( $data_rel_path ) {
			$path .= $data_rel_path;
			if ( substr( $path, -1 ) !== '/' ) {
				$path .= '/';
			}
		}

		return $path;
	}

	/**
	 * tempディレクトリのパスを取得
	 */
	protected function get_temp_dir_path() {
		return plugin_dir_path( __FILE__ ) . 'temp/';
	}

	/**
	 * cronのロックファイルのパスを取得
	 */
	protected function get_cron_lock_path() {
		return $this->get_data_dir_path() . 'cron_lock';
	}

	/**
	 * cronのステータスファイルのパスを取得 -- maruyama
	 */
	protected function get_cron_status_path() {
		return $this->get_data_dir_path() . 'cron_status';
	}

	/**
	 * cronのバックアップファイルのパスを取得 -- maruyama
	 */
	protected function get_cron_backup_path() {
		return $this->get_data_dir_path() . 'cron_backup';
	}

	/**
	 * プラグインのメンテナンスモードか判定
	 */
	protected function is_maintenance() {
		global $wp_filesystem;
		$maintenance_path = $this->get_temp_dir_path() . 'maintenance.php';
		if ( $wp_filesystem->exists( $maintenance_path ) ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * トラッキングIDを取得
	 */
	protected function get_tracking_id( $url = null ) {
		if ( $this->is_wordpress() ) {
			// auto
			$parse_url = parse_url( get_home_url() );
			$id = 'a&' . $parse_url['host'];
		} else {
			// manual
			$id .= 'm&';
		}
		return hash( 'fnv164', $id );
	}

	/**
	 * WPサイトか判定
	 * この判定方法で良いのかは要検証。今後変わる可能性あり
	 */
	protected function is_wordpress() {
		if ( function_exists( 'wp_nonce_field' ) ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * qahm対象ページか判定
	 * ※プラグイン読み込み直後のコンストラクタやwp_ajax内では$type引数指定無しの形は使えないので注意
	 */
	protected function is_qahm_page( $type = null ) {
		if ( $type ){
			if(
				$type === 'home' ||
				$type === 'page_id' ||
				$type === 'p' ||
				$type === 'cat' ||
				$type === 'tag' ||
				$type === 'tax'
			) {
				return true;
			} else {
				return false;
			}
		} else {
			if ( is_home() || is_front_page() || is_page() || is_single() || is_category() || is_tag() || is_tax() ) {
				return true;
			} else {
				return false;
			}
		}
	}

	/**
	 * ajaxに関数を登録
	 */
	protected function regist_ajax_func( $func ) {
		add_action( 'wp_ajax_' . QAHM_NAME . '_' . $func, array( $this, $func ) );
		add_action( 'wp_ajax_nopriv_' . QAHM_NAME . '_' . $func, array( $this, $func ) );
	}
	
	/**
	 * wp_filesystem 初期化
	 */
	public function init_wp_filesystem() {
		require_once ABSPATH . 'wp-admin/includes/file.php';
		global $wp_filesystem;

		if ( empty( $wp_filesystem ) ) {
			$creds = false;

			$access_type = get_filesystem_method();
			if ( $access_type === 'ftpext' ) {
				$creds = request_filesystem_credentials( '', '', false, false, null );
			}
			if ( ! WP_Filesystem( $creds ) ) {
				throw new Exception( 'WP_Filesystem Invalid Credentials' );
			}
		}
	}
	
	/**
	 * qa 翻訳関数
	 */
	public function qa_lang__( $text, $domain ) {
		return $text;
	}

	public function qa_lang_e( $text, $domain ) {
		echo $text;
	}
	
	public function qa_langesc_html__( $text, $domain ) {
		return esc_html( $text );
	}
	
	public function qa_langesc_html_e( $text, $domain ) {
		echo esc_html( $text );
	}

	public function qa_langesc_url__( $text, $domain ) {
		return esc_url( $text );
	}
	
	public function qa_langesc_url_e( $text, $domain ) {
		echo esc_url( $text );
	}

	public function qa_langesc_attr__( $text, $domain ) {
		return esc_attr( $text );
	}
	
	public function qa_langesc_attr_e( $text, $domain ) {
		echo esc_attr( $text );
	}	
}
