<?php
/**
 * QAHMで使用するデータのクラス
 * 
 * 全てのデータファイルは下記のデータを入れる
 * 
 * 1行目に404コード
 * - セキュリティ対策。名称は404
 * 
 * 2行目にヘッダー情報
 * - ひとつしかない情報を入れる。名称はheader
 * 
 * 3行目以降にデータ本体
 * - 複数存在する情報を入れる。名称はbody
 * 
 * を書き込む。
 * 
 * ※データのルール
 * - ヘッダーの先頭は必ずデータのバージョンとする。
 * - 区切り文字はtabにしてtsvの形式で扱う。
 * - セキュリティ対策のため拡張子は.phpにする。
 * 
 * バージョンの差異を吸収する関数も作る予定だが、規模が大きくなる可能性がある。
 * その時はより細分化する予定
 *
 * @package qa_heatmap
 */

class QAHM_File_Data extends QAHM_File_Base {

	// 全てのデータに共通する行番号の指定（column）
	const DATA_COLUMN_SECURITY = 0;
	const DATA_COLUMN_HEADER   = 1;
	const DATA_COLUMN_BODY     = 2;

	// 現在使用しているデータのバージョン
	const DATA_CURRENT_VERSION = array(
		'SESSION_TEMP'           => 1,
		'SESSION_FINISH'         => 1,
		'REALTIME_VIEW'          => 1,
		'POS'                    => 1,
		'CLICK'                  => 1,
		'EVENT'                  => 1,
		'MERGE_CLICK'            => 1,
		'MERGE_ATTENTION_SCROLL' => 1,
	);

	// 以下定数はデータに格納する順序を定義（row）

	/*
	 * ヘッダーの共通データ
	 * 
	 * データの中身を見る際、必ずバージョンを判定する必要がある。
	 * 定数名にはバージョン情報が含まれているため、
	 * まずはこの値でバージョンチェックし使用する定数を選ぶ必要がある。
	 * そのため各定数内にはバージョン情報を含めていない。
	 */ 
	const DATA_HEADER_VERSION = 0;

	// セッションデータ（temp） バージョン1
	const DATA_SESSION_TEMP_1 = array(
		// header
		'TRACKING_ID'       => 1,       // トラッキングID
		'DEVICE_NAME'       => 2,       // デバイス
		'IS_NEW_USER'       => 3,       // 新規ユーザー判定。1なら新規
		'USER_AGENT'        => 4,       // ユーザーエージェント
		'FIRST_REFERRER'    => 5,       // リファラ
		'UTM_SOURCE'        => 6,       // UTM_SOURCE
		'UTM_MEDIUM'        => 7,       // UTM_MEDIUM
		'UTM_CAMPAIGN'      => 8,       // UTM_CAMPAIGN
		'UTM_TERM'          => 9,       // UTM_TERM
		'ORIGINAL_ID'       => 10,      // オリジナルID

		// body
		'PAGE_URL'          => 0,       // URL
		'PAGE_TITLE'        => 1,       // タイトル
		'PAGE_TYPE'         => 2,       // タイプ
		'PAGE_ID'           => 3,       // ID
		'ACCESS_TIME'       => 4,       // 最初にアクセスした時刻（timestamp）
		'PAGE_SPEED'        => 5,       // document.readyが終わるまでの時間
	);

	// セッションデータ（finish） バージョン1
	const DATA_SESSION_FINISH_1 = array(
		// header
		'TRACKING_ID'       => 1,       // トラッキングID
		'DEVICE_NAME'       => 2,       // デバイス名
		'IS_NEW_USER'       => 3,       // 新規ユーザー判定。1なら新規
		'USER_AGENT'        => 4,       // ユーザーエージェント
		'FIRST_REFERRER'    => 5,       // リファラ
		'UTM_SOURCE'        => 6,       // UTM_SOURCE
		'UTM_MEDIUM'        => 7,       // UTM_MEDIUM
		'UTM_CAMPAIGN'      => 8,       // UTM_CAMPAIGN
		'UTM_TERM'          => 9,       // UTM_TERM
		'ORIGINAL_ID'       => 10,      // オリジナルID

		// body
		'PAGE_URL'          => 0,       // URL
		'PAGE_TITLE'        => 1,       // タイトル
		'PAGE_TYPE'         => 2,       // タイプ
		'PAGE_ID'           => 3,       // ID
		'ACCESS_TIME'       => 4,       // 最初にアクセスした時刻（timestamp）
		'PAGE_SPEED'        => 5,       // ページスピード（document.readyが終わるまでの時間）
		'TIME_ON_PAGE'      => 6,       // ページ滞在時間
	);

	// リアルタイムビューデータ バージョン1
	const DATA_REALTIME_VIEW_1 = array(
		// body
		'SESSION_FILE'      => 0,       // セッションファイル名
		'TRACKING_ID'       => 1,       // トラッキングID
		'DEVICE_NAME'       => 2,       // デバイス名
		'IS_NEW_USER'       => 3,       // 新規ユーザー判定
		'USER_AGENT'        => 4,       // ユーザーエージェント
		'FIRST_REFERRER'    => 5,       // リファラ
		'UTM_SOURCE'        => 6,       // UTM_SOURCE
		'UTM_MEDIUM'        => 7,       // UTM_MEDIUM
		'UTM_CAMPAIGN'      => 8,       // UTM_CAMPAIGN
		'UTM_TERM'          => 9,       // UTM_TERM
		'ORIGINAL_ID'       => 10,      // オリジナルID
		'FIRST_ACCESS_TIME' => 11,      // 流入ページの最初にアクセスした時刻（timestamp）
		'FIRST_URL'         => 12,      // 流入ページのURL
		'FIRST_TITLE'       => 13,      // 流入ページのタイトル
		'LAST_EXIT_TIME'    => 14,      // 離脱ページの離脱時刻（timestamp）
		'LAST_URL'          => 15,      // 離脱ページのURL
		'LAST_TITLE'        => 16,      // 離脱ページのタイトル
		'PV_NUM'            => 17,      // PV数
		'TIME_ON_SITE'      => 18,      // サイト滞在時間
	);

	// 位置データ バージョン1
	const DATA_POS_1 = array(
		// body
		'PERCENT_HEIGHT'    => 0,       // 高さを百分率で求めた値
		'TIME_ON_HEIGHT'    => 1,       // 高さあたりの滞在時間（秒）
	);

	// クリックデータ バージョン1
	const DATA_CLICK_1 = array(
		// body
		'SELECTOR_NAME'     => 0,       // セレクタ名
		'SELECTOR_X'        => 1,       // セレクタ左上からの相対座標X
		'SELECTOR_Y'        => 2,       // セレクタ左上からの相対座標Y
		'TRANSITION'        => 3,       // 遷移先のURL
	);

	// イベントデータ バージョン1
	const DATA_EVENT_1 = array(
		// header
		'WINDOW_INNER_W'    => 1,       // 解像度W
		'WINDOW_INNER_H'    => 2,       // 解像度H
		'DEVICE_NAME'       => 3,       // デバイス名 削除
		'COUNTRY'           => 4,       // 国 readersに移動するので削除

		// body
		'TYPE'              => 0,       // イベントタイプ
		'TIME'              => 1,       // イベントの発生時刻（読み込み完了からのms）
		'CLICK_X'           => 2,       // クリックイベントのX座標
		'CLICK_Y'           => 3,       // クリックイベントのY座標
		'SCROLL_Y'          => 2,       // スクロールイベントのY座標
		'MOUSE_X'           => 2,       // マウスの移動イベントのX座標
		'MOUSE_Y'           => 3,       // マウスの移動イベントのY座標
		'RESIZE_X'          => 2,       // リサイズイベントのX座標
		'RESIZE_Y'          => 3,       // リサイズイベントのY座標
	);

	// マージしたヒートマップデータ バージョン1
	const DATA_MERGE_CLICK_1 = array(
		// body
		'SELECTOR_NAME'     => 0,       // セレクタ名
		'SELECTOR_X'        => 1,       // セレクタ相対座標X
		'SELECTOR_Y'        => 2,       // セレクタ相対座標Y
	);

	// マージしたアテンションデータ バージョン1
	const DATA_MERGE_ATTENTION_SCROLL_1 = array(
		// body
		'PERCENT'           => 0,       // 100分率した番号
		'STAY_TIME'         => 1,       // 平均滞在時間（秒）
		'EXIT_USER'         => 2,       // 離脱した読者の数
	);



	/**
	 * 配列に格納された値をデータファイル用に最適化して文字列にして返す（add）
	 * この関数を用いて返された文字列は改行やタブをつけるタイミングも自動で行う
	 * そのため外部から改行コード（PHP_EOL）が挿入されていた場合は強制削除
	 */
	protected function get_file_data_string( $data_ary, $base_str = '' ) {
		if ( $base_str ) {
			$str = $base_str . PHP_EOL;
		} else {
			$str = '<?php http_response_code(404);exit; ?>' . PHP_EOL;
		}

		for ( $i = 0, $cnt = count( $data_ary ); $i < $cnt; $i++ ) {
			$replace = str_replace( PHP_EOL, '', $data_ary[ $i ] );
			$replace = str_replace( "\t", '', $replace );
			$str .= $replace;
			if ( $i !== $cnt - 1 ) {
				$str .= "\t";
			}
		}

		return $str;
	}


	/**
	 * tsv形式の文字列データを二次元配列に変換して返す
	 */
	protected function convert_tsv_to_array( $tsv ) {
		$tsv_ary = array();
		$tsv_col = explode( PHP_EOL, $tsv );

		foreach ( $tsv_col as $tsv_row ) {
			$tsv_row_ary = explode( "\t", $tsv_row );
			$tsv_ary[]   = $tsv_row_ary;
		}

		return $tsv_ary;
	}

	/**
	 * 二次元配列をtsv形式の文字列データに変換して返す
	 */
	protected function convert_array_to_tsv( $ary ) {
		$tsv = '';

		for ( $i = 0, $col_cnt = count( $ary ); $i < $col_cnt; $i++ ) {
			for ( $j = 0, $raw_cnt = count( $ary[$i] ); $j < $raw_cnt; $j++ ) {
				// 値にPHP_EOLや\tが入っていた場合はtsvの形が崩れる可能性があるので無視
				$replace = str_replace( PHP_EOL, '', $ary[ $i ][ $j ] );
				$replace = str_replace( "\t", '', $replace );
				$tsv .= $replace;

				if ( $j === $raw_cnt - 1 ) {
					if ( $i !== $col_cnt -1 ) {
						$tsv .= PHP_EOL;
					}
				} else {
					$tsv .= "\t";
				}
			}
		}

		return $tsv;
	}
}
